require 'date'
require_relative 'key_value_db'
require 'testhelper'

module Statistics
    $statistics_db = nil
    $pass_list = []
    $today = nil
    $this_month = nil
    TYPE_DAY = "day"
    TYPE_MONTH = "month"
    VERSION = "1.0.0"
    SAVE_DAYS = 32
    def self.set_db_file(db_file_path)
        $statistics_db = KeyValueDB.new(db_file_path, 999999)
    end
    def self.new_today_data(cur_time)
        today = {
            "id" => get_day_id(cur_time),
            "type" => TYPE_DAY,
            "version" => VERSION,
            "count" => 0,
            "ip" => {},
            "updated_time" => get_time_str(cur_time)
        }
    end
    def self.new_this_month_data(cur_time)
        this_month = {
            "id" => get_month_id(cur_time),
            "type" => TYPE_MONTH,
            "version" => VERSION,
            "count" => 0,
            "ip" => {},
            "updated_time" => get_time_str(cur_time)
        }
    end
    def self.set_filter_pass_list(pass_list)
        $pass_list = pass_list
    end

    def self.get_day_id(the_time=nil)
        if the_time == nil
            the_time = Time.now()
        end
        the_time.strftime('%Y%m%d')
    end
    def self.get_month_id(the_time=nil)
        if the_time == nil
            the_time = Time.now()
        end
        the_time.strftime('%Y%m')
    end
    def self.get_time_str(the_time=nil)
        if the_time == nil
            the_time = Time.now()
        end
        the_time.strftime('%Y-%m-%d %H:%M:%S.%6N')
    end
    def self.clear_historic_data(the_time)
        pre_day_time = the_time - SAVE_DAYS * 86400
        pre_day_id = get_day_id(pre_day_time)
        all_data = $statistics_db.get_data()
        all_data.reject! do |record|
            record["type"] == TYPE_DAY and record["id"] < pre_day_id
        end
        $statistics_db.set_data(all_data)
    end
    def self.get_today_and_this_month(the_time)
        if $today == nil and $this_month == nil
            $today = $statistics_db.find("id",get_day_id(the_time))
            if $today == nil
                $today = new_today_data(the_time)
            end
            $this_month =  $statistics_db.find("id",get_month_id(the_time))
            if $this_month == nil
                $this_month = new_this_month_data(the_time)
            end
        end
        if $today["id"] != get_day_id(the_time)
            $today = new_today_data(the_time)
        end
        if $this_month["id"] != get_month_id(the_time)
            $this_month = new_this_month_data(the_time)
        end
        [$today, $this_month]
    end

    def self.execute(request)
        puts request.path_info
        if not request.path_info.start_with?("/statistics") and 
                not $pass_list.include?(request.path_info)
            cur_time = Time.now()
            today, this_month = get_today_and_this_month(cur_time)

            today["count"] += 1
            today["updated_time"] = get_time_str(cur_time)
            if today["ip"].include?(request.ip)
                today["ip"][request.ip] += 1
            else
                today["ip"][request.ip] = 1
            end

            this_month["count"] += 1
            if this_month["ip"].include?(request.ip)
                this_month["ip"][request.ip] += 1
            else
                this_month["ip"][request.ip] = 1
            end
            this_month["updated_time"] = today["updated_time"]

            $statistics_db.add_or_update_by_key(today, "id")
            $statistics_db.add_or_update_by_key(this_month, "id")
            clear_historic_data(cur_time)
            $statistics_db.save()
        end
    end
end
get "/extension/statistics/month_list/:month_count" do |month_count|
    month_count = month_count.to_i
    if month_count > 0
        this_month = Date.today
        (2).upto(month_count).each {|x| this_month=this_month.prev_month}
        prev_month_str = this_month.to_s.gsub("-","")[0..5]
        all_data = $statistics_db.get_data()
        new_data = all_data.select do |record|
            # [record["id"], prev_month_str, (record["id"] >= prev_month_str)].ppt
            record["type"] == Statistics::TYPE_MONTH and record["id"] >= prev_month_str
        end
        new_data.to_json
    else
        all_data = $statistics_db.get_data()
        new_data = all_data.select do |record|
            # [record["id"], prev_month_str, (record["id"] >= prev_month_str)].ppt
            record["type"] == Statistics::TYPE_MONTH
        end
        new_data.to_json
    end
end
get "/extension/statistics/day_list_at_this_month" do 
    this_month = Date.today
    this_month_str = this_month.to_s.gsub("-","")[0..5]
    all_data = $statistics_db.get_data()
    new_data = all_data.select do |record|
        # [record["id"], prev_month_str, (record["id"] >= prev_month_str)].ppt
        record["type"] == Statistics::TYPE_DAY and record["id"].start_with?(this_month_str)
    end
    new_data.to_json
end